@@ -12,14 +12,14 @@ Allows access to all Salesforce objects accessible via the SOQL API.
 """
 import logging
 
-from salesforce.dbapi.exceptions import (  # NOQA pylint:disable=unused-import,useless-import-alias
+# Default version of Force.com API.
+# It can be customized by settings.DATABASES['salesforce']['API_VERSION']
+API_VERSION = '53.0'  # Winter '22
+
+from salesforce.dbapi.exceptions import (  # NOQA pylint:disable=unused-import,useless-import-alias,wrong-import-position
     IntegrityError as IntegrityError, DatabaseError as DatabaseError, SalesforceError as SalesforceError,
 )
 
-__version__ = "3.2"
+__version__ = "4.0"
 
 log = logging.getLogger(__name__)
-
-# Default version of Force.com API.
-# It can be customized by settings.DATABASES['salesforce']['API_VERSION']
-API_VERSION = '51.0'  # Spring '21
