@@ -12,27 +12,25 @@ Salesforce introspection code.  (like django.db.backends.*.introspection)
 import logging
 import re
 from collections import OrderedDict, namedtuple
-from typing import Any, Dict, List, Optional, Set, Tuple
+from typing import Any, Dict, Iterable, List, Optional, Set, Tuple
 
 from django.db.backends.base.introspection import (
     BaseDatabaseIntrospection, FieldInfo as BaseFieldInfo, TableInfo,
 )
 from django.utils.text import camel_case_to_spaces
 from django.db.backends.utils import CursorWrapper as _Cursor  # for typing
-# require "simplejson" to ensure that it is available to "requests" hook.
-import simplejson  # NOQA pylint:disable=unused-import
 
-from salesforce.backend import DJANGO_32_PLUS
+from salesforce.backend import DJANGO_22_PLUS, DJANGO_32_PLUS
 import salesforce.fields
 
 log = logging.getLogger(__name__)
 
-FieldInfo = namedtuple(  # type:ignore[misc]
+FieldInfo = namedtuple(  # type: ignore[misc]
     'FieldInfo',
     'name type_code display_size internal_size precision scale null_ok default'
     + (' collation' if DJANGO_32_PLUS else '') +
     ' params'  # the last name 'params' is our extension for Salesforce
-)  # pylint:disable=invalid-name
+)
 assert FieldInfo._fields[:-1] == BaseFieldInfo._fields
 
 # these sObjects are not tables - ignore them
@@ -46,6 +44,7 @@ PROBLEMATIC_OBJECTS = [
     'AsyncOperationEvent',  # new in API 46.0 Summer '19
     'AsyncOperationStatus',  # new in API 46.0 Summer '19
     'FlowExecutionErrorEvent',  # new in API 47.0 Winter '20 - missing 'Id'
+    'FlowOrchestrationEvent',  # new in API 53.0 Winter '22
 ]
 
 # this global variable is for `salesforce.management.commands.inspectdb`
@@ -109,19 +108,23 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         self._converted_lead_status = None  # type: Optional[str]
         self.is_tooling_api = False  # modified by other modules
 
-    def filter_table_list(self, table_names):
+    # -- custom methods
+
+    def filter_table_list(self, table_names: Iterable[str]):
+        # value "self._table_list_cache" is never None after avaluating the property "self.table_list_cache"
+        assert self.table_list_cache and self._table_list_cache
         self._table_list_cache['sobjects'] = [
             x for x in self.table_list_cache['sobjects'] if x['name'] in table_names
         ]
         self._table_names = self._table_names.intersection(table_names)
-        unknown_tables = [x for x in table_names if x not in self.connection.introspection._table_names]
+        unknown_tables = [x for x in table_names if x not in self.connection.introspection._table_names]  # noqa pylint:disable=protected-access
         if unknown_tables:
             raise ValueError('These tables are not a part of the inspected Salesforce database: '
                              '{}'.format(unknown_tables))
 
-    def reset_table_list(self, table_names):
+    def reset_table_list(self) -> None:
         self._table_list_cache = None
-        self.table_list_cache
+        self.table_list_cache  # pylint:disable=pointless-statement
 
     @property
     def sobjects_prefix(self) -> str:
@@ -151,18 +154,41 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
             self._table_description_cache[table] = response.json(object_pairs_hook=OrderedDict)
             field_list = self._table_description_cache[table]['fields']
             # 'Id' field is sometimes not the first field in tooling metadata SObjects
-            id_field, = [x for x in field_list if x['name'] == 'Id']
+            id_fields = [x for x in field_list if x['name'] == 'Id']
+            assert len(id_fields) == 1, "Table {!r} must contain one field with name 'Id'".format(table)
+            id_field, = id_fields
             assert id_field['type'] == 'id', (
                 "Invalid type of the field 'Id' in table '{}'".format(table))
             del field_list[field_list.index(id_field)]
         return self._table_description_cache[table]
 
+    # -- standard methods
+
+    if DJANGO_22_PLUS:
+
+        def identifier_converter(self, name: str) -> str:
+            """A conversion to the identifier for the purposes of comparison."""
+            return name.lower()
+
+    else:
+
+        def table_name_converter(self, name: str) -> str:  # pylint:disable=no-self-use
+            return name.lower()
+
+        def column_name_converter(self, name: str) -> str:  # pylint:disable=no-self-use
+            return name.lower()
+
     def get_table_list(self, cursor: _Cursor) -> List[TableInfo]:
         "Returns a list of table names in the current database."
         result = [TableInfo(SfProtectName(x['name']), 't') for x in self.table_list_cache['sobjects']]
         return result
 
-    def get_field_params(self, field: Dict[str, Any]) -> Dict[str, Any]:
+    # -- custom methods
+
+    def get_field_names(self, table_name: str) -> List[str]:
+        return [x['name'] for x in self.table_description_cache(table_name)['fields']]
+
+    def get_field_params(self, field: Dict[str, Any]) -> Dict[str, Any]:  # pylint:disable=too-many-branches
         params = OrderedDict()
         if field['label'] and field['label'] != camel_case_to_spaces(re.sub('__c$', '', field['name'])).title():
             params['verbose_name'] = field['label']
@@ -195,13 +221,15 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
                 params['ref_comment'] = 'References to missing tables: {}'.format(self.references_to(field, all=True))
         return params
 
-    def references_to(self, field: Dict[str, Any], all: bool = False) -> Optional[List[str]]:
+    def references_to(self, field: Dict[str, Any], all: bool = False) -> Optional[List[str]]:  # pylint:disable=redefined-builtin # noqa
         if field['type'] != 'reference':
             return None
         reference_to_valid = [x for x in field['referenceTo'] if x in self._table_names]
         reference_to_invalid = ['-{}'.format(x) for x in field['referenceTo'] if x not in self._table_names]
         return reference_to_valid + reference_to_invalid if all else reference_to_valid
 
+    # -- standard methods
+
     def get_table_description(self, cursor: _Cursor, table_name: str) -> List[FieldInfo]:
         "Returns a description of the table, with the DB-API cursor.description interface."
         # pylint:disable=too-many-locals,unused-argument
@@ -221,7 +249,7 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
             optional_kw = {'collation': None} if DJANGO_32_PLUS else {}
             # We prefer "length" over "byteLength" for "internal_size".
             # (because strings have usually: byteLength == 3 * length)
-            result.append(FieldInfo(  # type:ignore[call-arg] # problem with a conditional type
+            result.append(FieldInfo(  # type: ignore[call-arg] # problem with a conditional type
                 field['name'],       # name,
                 field['type'],       # type_code,
                 field['length'],     # display_size,
@@ -236,28 +264,19 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
             ))
         return result
 
-    def get_sequences(self, cursor: _Cursor, table_name: str, table_fields=()) -> List[Dict[str, str]]:
+    # unused because feature 'supports_table_check_constraints == False'
+    def get_sequences(self, cursor: _Cursor, table_name: str, table_fields=()  # pragma: no cover
+                      ) -> List[Dict[str, str]]:
         pk_col = self.get_primary_key_column(cursor, table_name) or 'Id'
         return [{'table': table_name, 'column': pk_col}]
 
-    # never used until real migrations will be supported
-    def get_key_columns(self, cursor: _Cursor, table_name: str) -> List[Tuple[str, str, str]]:
-        """
-            (column_name, referenced_table_name, referenced_column_name)
-        """
-        key_columns = []
-        for name, item in self.get_constraints(cursor, table_name).items():
-            if not item['primary_key']:
-                key_columns.append((name,) + item['foreign_key'])
-        return key_columns
-
     def get_relations(self, cursor: _Cursor, table_name: str) -> Dict[str, Tuple[str, 'SfProtectName']]:
         """
         Returns a dictionary of {field_index: (field_index_other_table, other_table)}
         representing all relationships to the given table. Indexes are 0-based.
         """
         # pylint:disable=global-statement,too-many-locals,too-many-nested-blocks,unused-argument
-        def table2model(table_name):
+        def table2model(table_name: str) -> str:
             return SfProtectName(table_name).title().replace(' ', '').replace('-', '')
         global last_introspection
         result = {}
@@ -302,6 +321,17 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
         )
         return result
 
+    # never used until real migrations will be supported
+    def get_key_columns(self, cursor: _Cursor, table_name: str) -> List[Tuple[str, str, str]]:  # pragma: no cover
+        """
+            (column_name, referenced_table_name, referenced_column_name)
+        """
+        key_columns = []
+        for name, item in self.get_constraints(cursor, table_name).items():
+            if not item['primary_key']:
+                key_columns.append((name,) + item['foreign_key'])
+        return key_columns
+
     def get_constraints(self, cursor: _Cursor, table_name: str) -> Dict[str, Dict[str, Any]]:
         """
         Retrieves any constraints or keys (unique, pk, fk, check, index)
@@ -330,6 +360,8 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
                 )
         return result
 
+    # -- custom methods
+
     def get_additional_meta(self, table_name: str) -> List[str]:
         item = [x for x in self.table_list_cache['sobjects'] if x['name'] == table_name][0]
         return ["verbose_name = %r" % item['label'],
@@ -346,8 +378,6 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
             self._converted_lead_status = cur.fetchone()[0]
         return self._converted_lead_status
 
-# pylint:disable=too-few-public-methods
-
 
 class SymbolicModelsName:
     """A symbolic name from the `models` module.
@@ -391,10 +421,6 @@ class SfProtectName(str):
         name = re.sub(r'__c$', '', self)   # fixed custom name
         return re.sub(r'([a-z0-9])(?=[A-Z])', r'\1_', name).title().replace('_', '')
 
-    @property
-    def name(self) -> 'SfProtectName':
-        return self
-
 
 reverse_models_names = dict((obj.value, obj) for obj in
                             [SymbolicModelsName(name) for name in
