@@ -16,7 +16,7 @@ This module requires a customized package django-stubs (django-salesforce-stubs)
 import sys
 from typing import Generic, Optional, TypeVar
 from django.db.models import manager, Model
-from django.db.models.query import QuerySet
+from django.db.models.query import QuerySet  # pylint:disable=unused-import
 
 from salesforce import router
 from salesforce.backend import query
@@ -26,7 +26,7 @@ _T = TypeVar("_T", bound=Model, covariant=True)
 
 class SalesforceManager(manager.Manager, Generic[_T]):
 
-    if sys.version_info[:2] < (3, 6):
+    if sys.version_info[:2] < (3, 6):  # Python 3.5 - remove soon
         # this is a fix for Generic type issue https://github.com/python/typing/issues/498
         __copy__ = None
 
