@@ -9,17 +9,19 @@
 Salesforce object query and queryset customizations.  (like django.db.models.query)
 """
 from typing import Dict, Generic, Iterable, List, NoReturn, Optional, TYPE_CHECKING, Type, TypeVar
-import typing
+import typing  # pylint:disable=unused-import
 
 from django.conf import settings
 from django.db import NotSupportedError, models
 from django.db.models import query as models_query, Model
+from django.db.models.sql import where as sql_where
 from django.db.utils import DEFAULT_DB_ALIAS
 import django
 
 from salesforce.backend.indep import get_sf_alt_pk
-from salesforce.backend import compiler, DJANGO_22_PLUS, DJANGO_30_PLUS
+from salesforce.backend import compiler, DJANGO_22_PLUS, DJANGO_30_PLUS, DJANGO_40_PLUS
 from salesforce.backend.models_sql_query import SalesforceQuery
+from salesforce.backend.operations import BULK_BATCH_SIZE
 from salesforce.router import is_sf_database
 import salesforce.backend.utils
 
@@ -35,6 +37,9 @@ if TYPE_CHECKING:
 #             self.query.get_columns()
 #         return self.query.cursor.rowcount
 
+if DJANGO_40_PLUS and not hasattr(sql_where.WhereNode, 'as_salesforce'):
+    setattr(sql_where.WhereNode, 'as_salesforce', compiler.SalesforceWhereNode.as_salesforce)
+
 
 class SalesforceQuerySet(models_query.QuerySet, Generic[_T]):
     """
@@ -50,7 +55,10 @@ class SalesforceQuerySet(models_query.QuerySet, Generic[_T]):
                  hints: Optional[Dict[str, Model]] = None
                  ) -> None:
         if query is None:
-            query = SalesforceQuery(model, where=compiler.SalesforceWhereNode)
+            if DJANGO_40_PLUS:
+                query = SalesforceQuery(model)
+            else:
+                query = SalesforceQuery(model, where=compiler.SalesforceWhereNode)
         super().__init__(model=model, query=query, using=using, hints=hints)
 
     def using(self, alias: Optional[str]) -> 'SalesforceQuerySet[_T]':
@@ -71,7 +79,7 @@ class SalesforceQuerySet(models_query.QuerySet, Generic[_T]):
         """
         return self.sf(query_all=True)
 
-    def simple_select_related(self, *fields: str) -> NoReturn:
+    def simple_select_related(self, *fields: str) -> NoReturn:  # pylint:disable=no-self-use
         raise NotSupportedError("Obsoleted method .simple_select_related(), use .select_related() instead")
 
     def bulk_create(self, objs: Iterable[_T], batch_size: Optional[int] = None, ignore_conflicts: bool = False
@@ -85,6 +93,15 @@ class SalesforceQuerySet(models_query.QuerySet, Generic[_T]):
                     x.pk = get_sf_alt_pk()
         return super().bulk_create(objs, batch_size=batch_size, **kwargs)
 
+    def bulk_update(self, objs: Iterable[Model], fields: 'typing.Collection[str]',  # pylint:disable=arguments-differ
+                    batch_size: Optional[int] = None, all_or_none: bool = None):
+        self.sf(all_or_none=all_or_none)
+        if batch_size is not None and batch_size < 0:
+            raise ValueError('Batch size must be a positive integer.')
+        batch_size = min(batch_size, BULK_BATCH_SIZE) if batch_size else BULK_BATCH_SIZE
+        for chunk in salesforce.backend.utils.chunked(objs, batch_size):
+            bulk_update_small(chunk, fields, all_or_none=all_or_none)
+
     def sf(self,
            query_all: Optional[bool] = None,
            all_or_none: Optional[bool] = None,
@@ -126,7 +143,8 @@ class SalesforceQuerySet(models_query.QuerySet, Generic[_T]):
             query.insert_values(fields, objs, raw=raw)
             return query.get_compiler(using=using).execute_sql(returning_fields)
     elif DJANGO_22_PLUS:
-        def _insert(self, objs, fields, return_id=False, raw=False, using=None, ignore_conflicts=False):  # type: ignore[misc] # noqa
+        def _insert(self, objs, fields, return_id=False, raw=False,  # type: ignore[misc] # noqa pylint:disable=arguments-differ
+                    using=None, ignore_conflicts=False):
             """
             Insert a new record for the given model. This provides an interface to
             the InsertQuery class and is how Model.save() is implemented.
@@ -139,7 +157,8 @@ class SalesforceQuerySet(models_query.QuerySet, Generic[_T]):
             query.insert_values(fields, objs, raw=raw)
             return query.get_compiler(using=using).execute_sql(return_id)
     else:
-        def _insert(self, objs, fields, return_id=False, raw=False, using=None):  # type: ignore[misc] # noqa
+        def _insert(self, objs, fields, return_id=False,  # type: ignore[misc] # pylint:disable=arguments-differ
+                    raw=False, using=None):
             self._for_write = True
             if using is None:
                 using = self.db
@@ -155,7 +174,7 @@ def bulk_update_small(objs: 'typing.Collection[models.Model]', fields: Iterable[
                       ) -> None:
     # simple implementation without "batch_size" parameter, but with "all_or_none"
     # and objects from mixed models can be updated by one request in the same transaction
-    assert len(objs) <= 200
+    assert len(objs) <= BULK_BATCH_SIZE
     records = []
     dbs = set()
     for item in objs:
@@ -165,7 +184,7 @@ def bulk_update_small(objs: 'typing.Collection[models.Model]', fields: Iterable[
         values['id'] = item.pk
         values['type_'] = item._meta.db_table
         records.append(values)
-        dbs.add(item._state.db)
+        dbs.add(item._state.db)  # pylint:disable=protected-access
     db = dbs.pop()
     if dbs or not is_sf_database(db):
         raise ValueError("All updated objects must be from the same Salesforce database.")
