@@ -12,7 +12,10 @@ import django
 from django.conf import settings
 
 from salesforce import models
-from salesforce.models import SalesforceModel as SalesforceModelParent
+if not getattr(settings, 'SF_EXAMPLE_EXTENDED_MODELS', False):
+    from salesforce.models import SalesforceModel as SalesforceModelParent
+else:
+    from salesforce.models_extend import SalesforceModel as SalesforceModelParent
 
 
 SALUTATIONS = [
@@ -29,6 +32,8 @@ INDUSTRIES = [
     'Telecommunications', 'Transportation', 'Utilities'
 ]
 
+# pylint:disable=invalid-str-returned  # return type of __str__ method
+
 
 # This class customizes `managed = True` for tests and does not disturbe SF
 class SalesforceModel(SalesforceModelParent):
@@ -43,7 +48,7 @@ class User(SalesforceModel):
     LastName = models.CharField(max_length=80)
     FirstName = models.CharField(max_length=40)
     IsActive = models.BooleanField(default=False)
-    UserType = models.CharField(max_length=80)
+    UserType = models.CharField(max_length=80, default='Standard')
 
 
 class AbstractAccount(SalesforceModel):
@@ -111,7 +116,7 @@ class PersonAccount(AbstractAccount):
         abstract = True
 
 
-if getattr(settings, 'PERSON_ACCOUNT_ACTIVATED', False):
+if getattr(settings, 'SF_EXAMPLE_PERSON_ACCOUNT_ACTIVATED', False):
     class Account(PersonAccount):  # pylint:disable=model-no-explicit-unicode
         pass
 else:
@@ -133,9 +138,12 @@ class Contact(SalesforceModel):
     # The `default=` with lambda function is easy readable, but can be
     # problematic with migrations in the future because it is not serializable.
     # It can be replaced by normal function.
-    owner = models.ForeignKey(User, on_delete=models.DO_NOTHING,
-                              default=models.DefaultedOnCreate(User),
-                              related_name='contact_owner_set')
+    if not getattr(settings, 'SF_EXAMPLE_SIMPLE_DEFAULTS', False):
+        owner = models.ForeignKey(User, on_delete=models.DO_NOTHING,
+                                  default=models.DefaultedOnCreate(User),
+                                  related_name='contact_owner_set')
+    if getattr(settings, 'SF_EXAMPLE_CUSTOM_INSTALLED', False):
+        vs = models.DecimalField(custom=True, unique=True, max_digits=10, decimal_places=0, blank=True, null=True)
 
     def __str__(self):
         return self.name
@@ -187,9 +195,10 @@ class Lead(SalesforceModel):
                                             sf_read_only=models.NOT_CREATEABLE)
     # Deleted object can be found only in querysets with "query_all" SF method.
     IsDeleted = models.BooleanField(default=False, sf_read_only=models.READ_ONLY)
-    owner = models.ForeignKey(User, on_delete=models.DO_NOTHING,
-                              default=models.DefaultedOnCreate(User),
-                              related_name='lead_owner_set')
+    if not getattr(settings, 'SF_EXAMPLE_SIMPLE_DEFAULTS', False):
+        owner = models.ForeignKey(User, on_delete=models.DO_NOTHING,
+                                  default=models.DefaultedOnCreate(User),
+                                  related_name='lead_owner_set')
     last_modified_by = models.ForeignKey(User, on_delete=models.DO_NOTHING, null=True,
                                          sf_read_only=models.READ_ONLY,
                                          related_name='lead_lastmodifiedby_set')
@@ -244,8 +253,7 @@ class ChargentOrder(SalesforceModel):
         managed = False  # can not be managed if it eventually could not exist
 
     Name = models.CharField(max_length=255, db_column='Name')
-    # example of automatically recognized name  db_column='ChargentOrders__Balance_Due__c'
-    Balance_Due = models.CharField(max_length=255)
+    Balance_Due = models.CharField(max_length=255, db_column='ChargentOrders__Balance_Due__c')
 
 
 class CronTrigger(SalesforceModel):
@@ -272,7 +280,7 @@ class SalesforceParentModel(SalesforceModel):
     # This is not a custom field because is not defined in a custom model.
     # The API name is therefore 'Name'.
     name = models.CharField(max_length=80)
-    last_modified_date = models.DateTimeField(sf_read_only=models.READ_ONLY)
+    last_modified_date = models.DateTimeField(sf_read_only=models.READ_ONLY, auto_now=True)
     # This model is not custom because it has not an explicit attribute
     # `custom = True` in Meta and also has not a `db_table` that ends with
     # '__c'.
@@ -281,7 +289,7 @@ class SalesforceParentModel(SalesforceModel):
         abstract = True
 
 
-class Note(models.Model):
+class Note(SalesforceModel):
     title = models.CharField(max_length=80)
     body = models.TextField(null=True)
     parent_id = models.CharField(max_length=18)
@@ -295,7 +303,7 @@ class Opportunity(SalesforceModel):
     )
     close_date = models.DateField()
     stage = models.CharField(max_length=255, db_column='StageName')  # e.g. "Prospecting"
-    created_date = models.DateTimeField(sf_read_only=models.READ_ONLY)
+    created_date = models.DateTimeField(sf_read_only=models.READ_ONLY, auto_now_add=True)
     amount = models.DecimalField(max_digits=18, decimal_places=2, blank=True, null=True)
     probability = models.DecimalField(
         max_digits=3, decimal_places=0, verbose_name='Probability (%)',
@@ -329,7 +337,7 @@ except ImportError:
     models_template = None
 
 
-class Organization(models.Model):
+class Organization(SalesforceModel):
     name = models.CharField(max_length=80, sf_read_only=models.NOT_CREATEABLE)
     division = models.CharField(max_length=80, sf_read_only=models.NOT_CREATEABLE, blank=True)
     organization_type = models.CharField(max_length=40, verbose_name='Edition',
@@ -374,18 +382,26 @@ class Test(SalesforceParentModel):
     """
     # This is a custom field because it is defined in the custom model.
     # The API name is therefore 'TestField__c'
-    test_text = models.CharField(max_length=40)
-    test_bool = models.BooleanField(default=False)
-    contact = models.ForeignKey(Contact, null=True, on_delete=models.DO_NOTHING)
+    test_text = models.CharField(max_length=40, db_column='TestText__c')
+    test_bool = models.BooleanField(default=False, db_column='TestBool__c')
+    contact = models.ForeignKey(Contact, null=True, on_delete=models.DO_NOTHING, custom=True)
 
     class Meta:
         custom = True
         db_table = 'django_Test__c'
 
 
+class TestDetail(SalesforceModel):
+    parent = models.ForeignKey(Test, models.DO_NOTHING, db_column='Parent__c', sf_read_only=models.NOT_UPDATEABLE)
+    contact = models.ForeignKey('Contact', models.DO_NOTHING, db_column='Contact__c', blank=True, null=True)
+
+    class Meta(SalesforceModel.Meta):
+        db_table = 'django_Test_detail__c'
+
+
 # example of relationship from builtin object to custom object
 
-class Attachment(models.Model):
+class Attachment(SalesforceModel):
     # A standard SFDC object that can have a relationship to any custom object
     name = models.CharField(max_length=80)
     parent = models.ForeignKey(Test, sf_read_only=models.NOT_UPDATEABLE, on_delete=models.DO_NOTHING)
@@ -393,7 +409,7 @@ class Attachment(models.Model):
     body = models.TextField()
 
 
-class Task(models.Model):
+class Task(SalesforceModel):
     # Reference to tables [Contact, Lead]
     who = models.ForeignKey(Lead, on_delete=models.DO_NOTHING, blank=True, null=True)
     # Refer
@@ -402,7 +418,7 @@ class Task(models.Model):
 
 # OneToOneField
 
-class ApexEmailNotification(models.Model):
+class ApexEmailNotification(SalesforceModel):
     """Stores who should be notified when unhandled Apex exceptions occur.
 
     The target can be more Salesforce users or an external email addresses.
@@ -415,6 +431,18 @@ class ApexEmailNotification(models.Model):
     email = models.CharField(unique=True, max_length=255, verbose_name='email', blank=True)
 
 
-class Campaign(models.Model):
+class Campaign(SalesforceModel):
     name = models.CharField(max_length=80)
     number_sent = models.DecimalField(max_digits=18, decimal_places=0, verbose_name='Num Sent', blank=True, null=True)
+    parent = models.ForeignKey('Campaign', on_delete=models.DO_NOTHING, blank=True, null=True)
+
+
+class CampaignMember(SalesforceModel):
+    campaign = models.ForeignKey(Campaign, on_delete=models.DO_NOTHING)
+    contact = models.ForeignKey(Contact, on_delete=models.DO_NOTHING)
+    status = models.CharField(max_length=40)
+
+
+# this model will be removed to test removing in migrations
+class DeletedObject(SalesforceModel):
+    pass
