@@ -1,15 +1,18 @@
 """
 Tests that do not need to connect servers
 """
-# pylint:disable=deprecated-method,too-many-ancestors,unused-variable
+# pylint:disable=unused-variable
 
+from typing import Type
 from django.apps.registry import Apps
 from django.test import TestCase
 from django.db.models import DO_NOTHING, Subquery
 from salesforce import fields, models
+from salesforce.dbapi import driver
 from salesforce.testrunner.example.models import (
         Contact, Opportunity, OpportunityContactRole, ChargentOrder)
-from salesforce.backend.test_helpers import LazyTestMixin
+from salesforce.backend.test_helpers import default_is_sf, LazyTestMixin, skipUnless
+from salesforce.backend.utils import sobj_id
 
 
 class EasyCharField(models.CharField):
@@ -117,6 +120,7 @@ class TestQueryCompiler(TestCase, LazyTestMixin):
                          r'\(SELECT Opportunity\.Id FROM Opportunity WHERE Opportunity\.StageName = %s ?\)')
         self.assertRegex(sql, 'OpportunityContactRole.Role = %s')
 
+    @skipUnless(default_is_sf, "depends on Salesforce database.")
     def test_subquery_filter_on_child(self):
         """Filter with a Subquery() on a child object.
 
@@ -148,6 +152,7 @@ class TestTopologyCompiler(TestCase):
         ret = compiler.query_topology(alias_map_items)
         self.assertEqual(ret, expected)
 
+    @skipUnless(default_is_sf, "depends on Salesforce database.")
     def test_topology_compiler(self):
         # Contact.objects.all()
         # SELECT Contact.Id FROM Contact
@@ -194,3 +199,37 @@ class SfParamsTest(TestCase):
         # a valua is propagated to the next level, but not to the previous
         self.assertTrue(qs_3.query.sf_params.query_all)
         self.assertFalse(qs_1.query.sf_params.query_all)
+
+
+class RegisterConversionTest(TestCase):
+    @staticmethod
+    def unregister_conversion(type_: Type) -> None:
+        del driver.json_conversions[type_]
+        del driver.sql_conversions[type_]
+        if type_ in driver.subclass_conversions:
+            driver.subclass_conversions.remove(type_)
+
+    def test_arg_model_conversion(self) -> None:
+        contact = Contact(last_name='test contact', pk='003001234567890AAA')
+
+        self.assertNotIn(models.SalesforceModel, driver.json_conversions)
+        self.assertNotIn(models.SalesforceModel, driver.sql_conversions)
+        self.assertNotIn(models.SalesforceModel, driver.subclass_conversions)
+
+        driver.register_conversion(models.SalesforceModel,
+                                   json_conv=sobj_id,
+                                   sql_conv=lambda x: driver.quoted_string_literal(sobj_id(x)),
+                                   subclass=True)
+
+        self.assertEqual(driver.arg_to_json(contact), contact.pk)
+        self.assertEqual(driver.arg_to_soql(contact), contact.pk)
+
+        self.assertIn(models.SalesforceModel, driver.json_conversions)
+        self.assertIn(models.SalesforceModel, driver.sql_conversions)
+        self.assertIn(models.SalesforceModel, driver.subclass_conversions)
+
+        self.unregister_conversion(models.Model)
+
+        self.assertNotIn(models.SalesforceModel, driver.json_conversions)
+        self.assertNotIn(models.SalesforceModel, driver.sql_conversions)
+        self.assertNotIn(models.SalesforceModel, driver.subclass_conversions)
